@@ -12,6 +12,7 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 /* GTK */
 #include <gtk/gtk.h>
@@ -23,23 +24,50 @@
 #include "util.h"
 #include "sound.h"
 #include "callerid.h"
+#include "llcheck.h"
+
+/* graphical symbols */
+#include "hangup.xpm"
+#include "pickup.xpm"
 
 /* call timeout_callback each <this number> of milliseconds */
 #define TIMER_DELAY 300
 
 volatile int interrupted = 0; /* the caught signal will be stored here */
 
-/* the quit (e.g. menu entry) callback
+/*
+ * the quit (e.g. menu entry) callback
  * prototype is this way to standardize to GtkItemFactoryCallback2
  *
  * this function will always be called when gtk is to finish
+ *
+ * input: data: session
  */
 static void quit(GtkWidget *widget, gpointer data, guint action) {
+  struct session_t *session = (struct session_t *) data;
+
   gtk_handle_hang_up_button(NULL, data); /* simulate hang_up_button */
 
+  /* some (GUI) de-initialization, not directly session related */
+  llcheck_bar_deinit(session->llcheck_in);
+  llcheck_bar_deinit(session->llcheck_out);
+
   gtk_main_quit();
 }
 
+/*
+ * main window delete_event callback
+ *
+ * input: data: session
+ */
+gint delete_event(GtkWidget *widget,
+		  GdkEvent *event,
+		  gpointer data)
+{
+  quit(widget, data, 0);
+  return FALSE; /* continue event handling */
+}
+
 /* handler for SIGTERM and SIGINT */
 void terminate_signal_callback(int sig) {
   interrupted = sig;
@@ -49,6 +77,8 @@ void terminate_signal_callback(int sig) {
  * periodically (e.g. each 300 milliseconds) from gtk main loop called function
  */
 gint timeout_callback(gpointer data) {
+  struct session_t *session = (struct session_t *) data;
+
   if (interrupted) {
     switch(interrupted) {
     case SIGINT:
@@ -64,6 +94,23 @@ gint timeout_callback(gpointer data) {
     }
     quit(NULL, data, 0);
   }
+
+  if (session->state == STATE_CONVERSATION) {
+    char *timediff = timediff_str(time(NULL), session->vcon_time);
+    char buf[64];
+
+    if (snprintf(buf, 64, "%s %s",
+		 state_data[STATE_CONVERSATION].status_bar, timediff) >= 64)
+      buf[63] = '\0';
+    
+    gtk_statusbar_pop(GTK_STATUSBAR(session->status_bar),
+		      session->phone_context_id);
+    gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
+		       session->phone_context_id,
+		       buf);
+    free(timediff);
+  }
+  
   return TRUE; /* call it again */
 }
 
@@ -82,9 +129,10 @@ gint timeout_callback(gpointer data) {
  *       and maybe want to make it modal with
  *         gtk_window_set_modal(GTK_WINDOW(window), TRUE);
  */
-GtkWidget *get_ok_dialog(char *title, char *contents,
+GtkWidget *ok_dialog_get(char *title, char *contents,
 			 GtkJustification justification) {
   GtkWidget *window;
+  GtkWidget *button_box;
   GtkWidget *button;
   GtkWidget *label;
 
@@ -101,9 +149,14 @@ GtkWidget *get_ok_dialog(char *title, char *contents,
   gtk_widget_show(label);
 
   /* action area */
+  button_box = gtk_hbutton_box_new();
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->action_area),
+		    button_box);
+  gtk_widget_show(button_box);
+
+  /* OK button */
   button = gtk_button_new_with_label("OK");
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button,
-		     TRUE, TRUE, 0);
+  gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
 			    GTK_OBJECT(window));
@@ -190,7 +243,7 @@ static void cb_settings_ok(GtkWidget *widget, gpointer data) {
       gtk_widget_destroy(window);
     }
   } else { /* settings error -> restored old settings -> display note */
-    ok_window = get_ok_dialog("Note",
+    ok_window = ok_dialog_get("Note",
 			      "Bad option, please try again.",
 			      GTK_JUSTIFY_CENTER);
     gtk_window_set_modal(GTK_WINDOW(ok_window), TRUE);
@@ -203,6 +256,80 @@ static void cb_settings_ok(GtkWidget *widget, gpointer data) {
   free(old_audio_device_name_out);
 }
 
+/*
+ * File Menu entries
+ */
+
+struct info_row_t {
+  gchar *tag;
+  gchar *value;
+};
+
+/* Info window */
+static void cb_info_window(GtkWidget *widget, gpointer data, guint action) {
+  struct session_t *session = (struct session_t *) data;
+  struct info_row_t rows[] = {
+    { "Sound input device:", strdup(session->audio_device_name_in)},
+    { "Input speed:", ltostr(session->audio_speed_in) },
+    { "Input sample size:", ltostr(session->audio_sample_size_in * 8) },
+    { "Input channels:", ltostr(1) },
+    { "Sound output device:", strdup(session->audio_device_name_out)},
+    { "Output speed:", ltostr(session->audio_speed_out) },
+    { "Output sample size:", ltostr(session->audio_sample_size_out * 8) },
+    { "Output channels:", ltostr(1) },
+    { "ISDN device:", strdup(session->isdn_device_name) }
+  };
+  int i;
+
+  GtkWidget *window; /* the window itself, actually a GtkDialog */
+  GtkWidget *table;  /* for tag->value pairs */
+  GtkWidget *label;
+  GtkWidget *button_box;
+  GtkWidget *button;
+
+  window = gtk_dialog_new();
+  gtk_window_set_title(GTK_WINDOW(window), "Info");
+
+  /* the info area */
+  table = gtk_table_new(sizeof(rows) / sizeof(struct info_row_t), 2, FALSE);
+  gtk_container_set_border_width(GTK_CONTAINER(table), 10);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->vbox), table);
+  gtk_widget_show(table);
+
+  for (i = 0; i < sizeof(rows) / sizeof(struct info_row_t); i++) {
+    label = gtk_label_new(rows[i].tag);
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i+1);
+    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+    gtk_widget_show(label);
+
+    label = gtk_label_new(rows[i].value);
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i+1);
+    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+    gtk_widget_show(label);
+    
+    free(rows[i].value); /* free stdrup() mem */
+  }
+
+  /* action area */
+  button_box = gtk_hbutton_box_new();
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->action_area),
+		    button_box);
+  gtk_widget_show(button_box);
+
+  button = gtk_button_new_with_label("OK");
+  gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
+  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
+			    GTK_OBJECT(window));
+  gtk_widget_show(button);
+  
+  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
+
+  /* show everything */
+  gtk_widget_show(window);
+}
+
 /*
  * Options menu entries
  */
@@ -210,11 +337,12 @@ static void cb_settings_ok(GtkWidget *widget, gpointer data) {
 /* Settings menu entry callback */
 static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
   struct session_t *session = (struct session_t *) data;
-  GtkWidget *window;/* the dialog window itself */
-  GtkWidget *button;/* action area buttons */
-  GtkWidget *table; /* tables in frames */
-  GtkWidget *label; /* different labels */
-  GtkWidget *frame; /* the frames to separate msn/msns from sound devices */
+  GtkWidget *window; /* the dialog window itself */
+  GtkWidget *button_box; /* box for buttons in standard way */
+  GtkWidget *button; /* action area buttons */
+  GtkWidget *table;  /* tables in frames */
+  GtkWidget *label;  /* different labels */
+  GtkWidget *frame;  /* the frames to separate msn/msns from sound devices */
 
   GtkWidget *msn_entry;
   GtkWidget *msns_entry;
@@ -240,6 +368,7 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
 
   label = gtk_label_new("Identifying MSN:");
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,0,1);
+  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_show(label);
 
   msn_entry = gtk_entry_new();
@@ -250,6 +379,7 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
 
   label = gtk_label_new("Listen to MSNs:");
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
+  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_show(label);
 
   msns_entry = gtk_entry_new();
@@ -272,6 +402,7 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
 
   label = gtk_label_new("Input sound device:");
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,0,1);
+  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_show(label);
 
   audio_device_name_in_entry = gtk_entry_new();
@@ -283,6 +414,7 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
 
   label = gtk_label_new("Output sound device:");
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
+  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_show(label);
 
   audio_device_name_out_entry = gtk_entry_new();
@@ -293,9 +425,13 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
   gtk_widget_show(audio_device_name_out_entry);
 
   /* action area */
+  button_box = gtk_hbutton_box_new();
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->action_area),
+  		    button_box);
+  gtk_widget_show(button_box);
+
   button = gtk_button_new_with_label("OK");
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button,
-		     TRUE, TRUE, 0);
+  gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
   gtk_object_set_data(GTK_OBJECT(button), "msn_entry", (gpointer) msn_entry);
   gtk_object_set_data(GTK_OBJECT(button), "msns_entry", (gpointer) msns_entry);
   gtk_object_set_data(GTK_OBJECT(button), "audio_device_name_in_entry",
@@ -309,8 +445,7 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
   gtk_widget_show(button);
 
   button = gtk_button_new_with_label("Cancel");
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button,
-		     TRUE, TRUE, 0);
+  gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
 			    GTK_OBJECT(window));
@@ -328,7 +463,7 @@ static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
 
 /* the about menu entry callback */
 static void cb_about(GtkWidget *widget, gpointer data, guint action) {
-  GtkWidget *window = get_ok_dialog("About ANT",
+  GtkWidget *window = ok_dialog_get("About ANT",
 			"ANT (ANT is Not a Telephone) Version " VERSION "\n"
 			"Copyright 2002 Roland Stigge\n\n"
 			"This is a small ISDN telephone application\n"
@@ -342,7 +477,7 @@ static void cb_about(GtkWidget *widget, gpointer data, guint action) {
 
 /* the about menu entry callback */
 static void cb_license(GtkWidget *widget, gpointer data, guint action) {
-  GtkWidget *window = get_ok_dialog("ANT License",
+  GtkWidget *window = ok_dialog_get("ANT License",
     "ANT (ANT is Not a Telephone)\n"
     "Copyright (C) 2002 Roland Stigge\n"
     "\n"
@@ -359,20 +494,11 @@ static void cb_license(GtkWidget *widget, gpointer data, guint action) {
     "You should have received a copy of the GNU General Public License\n"
     "along with this program; if not, write to the Free Software\n"
     "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, "
-    "USA.", GTK_JUSTIFY_LEFT);
+    "USA.", GTK_JUSTIFY_CENTER);
 
   gtk_widget_show(window);
 }
 
-/* main window delete_event callback */
-gint delete_event(GtkWidget *widget,
-		  GdkEvent *event,
-		  gpointer data)
-{
-  quit(widget, data, 0);
-  return FALSE; /* continue event handling */
-}
-
 /*
  * get the main menu widget
  *
@@ -385,13 +511,23 @@ GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
   /* The main menu structure */
   GtkItemFactoryEntry main_menu_items[] = {
     /*path                  accel.    callb.         cb param. kind          */
-    { "/_Phone",            NULL,     NULL,          0,        "<Branch>"    },
+    { "/Phon_e",            NULL,     NULL,          0,        "<Branch>"    },
+    { "/Phone/_Info Window",NULL,     cb_info_window,0,        NULL          },
+    { "/Phone/_Line Level Check",
+                            NULL,     llcheck,       0,        NULL,         },
+    { "/Phone/Sep",         NULL,     NULL,          0,        "<Separator>" },
     { "/Phone/_Quit",       "<alt>X", quit,          0,        NULL          },
-    { "/_Options",          NULL,     NULL,          0,        "<Branch>"    },
-    { "/Options/_Caller ID Monitor",
+
+    { "/_View",             NULL,     NULL,          0,        "<Branch>"    },
+
+    { "/View/_Caller ID Monitor",
                             NULL,     cid_toggle_cb, 0,        "<CheckItem>" },
-    { "/Options/Sep",       NULL,     NULL,          0,        "<Separator>" },
+    { "/View/_Line Level Meters",
+                            NULL,     llcheck_toggle_cb,0,     "<CheckItem>" },
+
+    { "/_Options",          NULL,     NULL,          0,        "<Branch>"    },
     { "/Options/_Settings", NULL,     cb_settings,   0,        NULL          },
+
     { "/_Help",             NULL,     NULL,          0,        "<LastBranch>"},
     { "/Help/_About",       NULL,     cb_about,      0,        NULL          },
     { "/Help/_License",     NULL,     cb_license,    0,        NULL          },
@@ -411,9 +547,18 @@ GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
   gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
   
   /* save connection to menu item(s), set defaults */
-  session->cid_check_menu_item = gtk_item_factory_get_item(item_factory, "/Options/Caller ID Monitor");
+  session->cid_check_menu_item = gtk_item_factory_get_item(item_factory,
+				   "/View/Caller ID Monitor");
+  session->llcheck_check_menu_item = gtk_item_factory_get_item(item_factory,
+				   "/View/Line Level Meters");
+  session->menuitem_settings = gtk_item_factory_get_item(item_factory,
+				   "/Options/Settings");
+  session->menuitem_line_check = gtk_item_factory_get_item(item_factory,
+				   "/Phone/Line Level Check");
   gtk_check_menu_item_set_active(
     GTK_CHECK_MENU_ITEM(session->cid_check_menu_item), TRUE);
+  gtk_check_menu_item_set_active(
+    GTK_CHECK_MENU_ITEM(session->llcheck_check_menu_item), TRUE);
   
   return gtk_item_factory_get_widget(item_factory, "<main>");
 }
@@ -427,12 +572,17 @@ GtkWidget *get_dial_box(struct session_t *session) {
   GtkWidget *frame;
   GtkWidget *hbox;
   GtkWidget *label;
+  GtkWidget *label_hbox;
+  GdkPixmap *pixmap;
+  GdkBitmap *mask;
+  GtkStyle *style;
+  GtkWidget *pixmapwid;
 
   frame = gtk_frame_new("Dialing");
   gtk_container_set_border_width(GTK_CONTAINER(frame), 8);
 
   /* dial hbox */
-  hbox = gtk_hbox_new(FALSE, 5);
+  hbox = gtk_hbox_new(FALSE, 10);
   gtk_container_add(GTK_CONTAINER(frame), hbox);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 8);
   gtk_widget_show(hbox);
@@ -442,14 +592,14 @@ GtkWidget *get_dial_box(struct session_t *session) {
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);
 
-  /* dial number entry */
+  /* dial number combo box */
   session->dial_number_box = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(session->dial_number_box),
 				session->dial_number_history);
   gtk_combo_set_use_arrows_always(GTK_COMBO(session->dial_number_box), TRUE);
-  gtk_object_set(GTK_OBJECT(session->dial_number_box), "width", 150, NULL);
+  gtk_object_set(GTK_OBJECT(session->dial_number_box), "width", 180, NULL);
   gtk_box_pack_start(GTK_BOX(hbox), session->dial_number_box,
-		     FALSE, FALSE, 0);
+		     TRUE, TRUE, 0);
   gtk_widget_show(session->dial_number_box);
   gtk_signal_disconnect(GTK_OBJECT(GTK_COMBO(session->dial_number_box)->entry),
 			GTK_COMBO(session->dial_number_box)->activate_id);
@@ -460,7 +610,7 @@ GtkWidget *get_dial_box(struct session_t *session) {
   gtk_signal_connect(GTK_OBJECT(session->pick_up_button), "clicked",
 		     GTK_SIGNAL_FUNC(gtk_handle_pick_up_button),
 		     (gpointer) session);
-  gtk_object_set(GTK_OBJECT(session->pick_up_button), "width", 60, NULL);
+  gtk_object_set(GTK_OBJECT(session->pick_up_button), "width", 82, NULL);
   /* XXX: set automatically to maximum possibly needed size */
   gtk_widget_show(session->pick_up_button);
 
@@ -471,10 +621,27 @@ GtkWidget *get_dial_box(struct session_t *session) {
 			    GTK_SIGNAL_FUNC(gtk_button_clicked),
 			    GTK_OBJECT(session->pick_up_button));
 
+  /* pick up button hbox */
+  label_hbox = gtk_hbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(session->pick_up_button),
+		    label_hbox);
+  gtk_widget_show(label_hbox);
+
+  /* pick up button symbol */
+  style = gtk_widget_get_style(session->main_window);
+  pixmap = gdk_pixmap_create_from_xpm_d(session->main_window->window,
+					&mask,
+					&style->bg[GTK_STATE_NORMAL],
+					(gchar **) pickup_xpm);
+
+  pixmapwid = gtk_pixmap_new(pixmap, mask);
+  gtk_box_pack_start(GTK_BOX(label_hbox), pixmapwid, FALSE, FALSE, 2);
+  gtk_widget_show(pixmapwid);
+  
   /* pick up button label */
   session->pick_up_label = gtk_label_new(NULL);
-  gtk_container_add(GTK_CONTAINER(session->pick_up_button),
-		    session->pick_up_label);
+  gtk_box_pack_start(GTK_BOX(label_hbox), session->pick_up_label,
+		     TRUE, FALSE, 3); /* expand but fill up outside label */
   gtk_widget_show(session->pick_up_label);
 
    /* hang up button */
@@ -483,14 +650,33 @@ GtkWidget *get_dial_box(struct session_t *session) {
   gtk_signal_connect(GTK_OBJECT(session->hang_up_button), "clicked",
 		     GTK_SIGNAL_FUNC(gtk_handle_hang_up_button),
 		     (gpointer) session);
-  gtk_object_set(GTK_OBJECT(session->hang_up_button), "width", 60, NULL);
+  gtk_object_set(GTK_OBJECT(session->hang_up_button), "width", 82, NULL);
   /* XXX: set automatically to maximum possibly needed size */
   gtk_widget_show(session->hang_up_button);
 
+  /* pick up button hbox */
+  label_hbox = gtk_hbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(session->hang_up_button),
+		    label_hbox);
+  gtk_widget_show(label_hbox);
+
+  /* pick up button symbol*/
+  /* style = gtk_widget_get_style(session->main_window); */
+    /* (already done for pickup button) */
+
+  pixmap = gdk_pixmap_create_from_xpm_d(session->main_window->window,
+					&mask,
+					&style->bg[GTK_STATE_NORMAL],
+					(gchar **) hangup_xpm);
+
+  pixmapwid = gtk_pixmap_new(pixmap, mask);
+  gtk_box_pack_start(GTK_BOX(label_hbox), pixmapwid, FALSE, FALSE, 2);
+  gtk_widget_show(pixmapwid);
+  
   /* hang up button label */
   session->hang_up_label = gtk_label_new(NULL);
-  gtk_container_add(GTK_CONTAINER(session->hang_up_button),
-		    session->hang_up_label);
+  gtk_box_pack_start(GTK_BOX(label_hbox), session->hang_up_label,
+		     TRUE, FALSE, 3); /* expand but fill up outside label */
   gtk_widget_show(session->hang_up_label);
   
   return frame;
@@ -510,19 +696,25 @@ int main_gtk(struct session_t *session) {
   GtkWidget *cidbox;
 
   /* the main window */
-  main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  session->main_window = main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(main_window), "ANT");
 
   gtk_signal_connect(GTK_OBJECT(main_window), "delete_event",
 		     GTK_SIGNAL_FUNC(delete_event), (gpointer) session);
 
   gtk_container_set_border_width(GTK_CONTAINER(main_window), 0);
+  gtk_widget_realize(main_window);
+
   gtk_object_set(GTK_OBJECT(main_window), "auto_shrink", TRUE, NULL);
   gtk_object_set(GTK_OBJECT(main_window), "allow_shrink", FALSE, NULL);
-  gtk_object_set(GTK_OBJECT(main_window), "allow_grow", FALSE, NULL);
+  gtk_object_set(GTK_OBJECT(main_window), "allow_grow", TRUE, NULL);
+
+  /* set some defaults */
+  gtk_hbutton_box_set_spacing_default(16); /* space between buttons */
+  gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_SPREAD);
 
   /* main vbox */
-  main_vbox = gtk_vbox_new(FALSE, 0); /* not homogeneous, spacing = 0*/
+  main_vbox = gtk_vbox_new(FALSE, 0); /* not homogeneous, spacing = 0 */
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 0);
   gtk_container_add(GTK_CONTAINER(main_window), main_vbox);
   gtk_widget_show(main_vbox);
@@ -535,7 +727,6 @@ int main_gtk(struct session_t *session) {
 
   /* the caller id section */
   cidbox = cid_get(session);
-  gtk_widget_show(cidbox);
 
   /* status bar */
   session->status_bar = gtk_statusbar_new();
@@ -545,19 +736,41 @@ int main_gtk(struct session_t *session) {
   gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
 		     session->phone_context_id, "");
 
+  /* line level check inside status bar */
+  session->llcheck = gtk_vbox_new(FALSE, 0);
+  gtk_box_pack_end(GTK_BOX(session->status_bar),
+		   session->llcheck, FALSE, FALSE, 0);
+
+  session->llcheck_in =
+    llcheck_bar_new(66, 10, 200, 0, 0);
+  gtk_box_pack_start(GTK_BOX(session->llcheck),
+		   session->llcheck_in, FALSE, FALSE, 0);
+  gtk_widget_show(session->llcheck_in);
+  session->llcheck_out =
+    llcheck_bar_new(66, 10, 0, 200, 0);
+  gtk_box_pack_start(GTK_BOX(session->llcheck),
+		   session->llcheck_out, FALSE, FALSE, 0);
+  gtk_widget_show(session->llcheck_out);
+
   /* main menu bar */
   main_menu = get_main_menu(main_window, session); /* main menu */
   gtk_widget_show(main_menu);
 
-  /* show them */
+  /* pack items into main vbox (after main menu bar creation) */
   gtk_box_pack_start(GTK_BOX(main_vbox), main_menu, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(main_vbox), dialbox, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(main_vbox), cidbox, FALSE, FALSE, 0);
-  gtk_box_pack_end(GTK_BOX(main_vbox), session->status_bar, TRUE, TRUE, 2);
+  gtk_box_pack_start(GTK_BOX(main_vbox), cidbox, TRUE, TRUE, 0);
+  gtk_box_pack_end(GTK_BOX(main_vbox), session->status_bar, FALSE, FALSE, 2);
+
+  /* show items on demand */
+  if (GTK_CHECK_MENU_ITEM (session->cid_check_menu_item)->active)
+    gtk_widget_show(cidbox);
+  if (GTK_CHECK_MENU_ITEM (session->llcheck_check_menu_item)->active)
+    gtk_widget_show(session->llcheck);
+
+  /* show everything */
+  gtk_widget_show(main_window);
 
-  /* grab focus at last */
-  gtk_widget_grab_focus(GTK_WIDGET(GTK_COMBO(session->dial_number_box)
-				   ->entry));
   /* set up for initial state */
   session_set_state(session, session->state); /* state is already in session */
 
@@ -569,9 +782,6 @@ int main_gtk(struct session_t *session) {
   signal(SIGINT, &terminate_signal_callback);
   signal(SIGTERM, &terminate_signal_callback);
 
-  /* show everything */
-  gtk_widget_show(main_window);
-
   /* gtk main loop */
   gtk_main();
   
