@@ -23,23 +23,13 @@
 #include "gtk.h"
 #include "util.h"
 #include "callerid.h"
+#include "llcheck.h"
 
 /*
  * This is our session. Currently just one globally.
  */
 struct session_t session;
 
-/*
- * Data needed for setting the session state (the state is the index)
- */
-struct state_data_t {
-  char *status_bar;
-  char *pick_up_label;
-  int pick_up_state;
-  char *hang_up_label;
-  int hang_up_state;
-};
-
 struct state_data_t state_data[] = {
   { "Ready",          "Dial",    1, "Hang up", 0 }, /* STATE_READY        */
   { "RING",           "Answer",  1, "Reject",  1 }, /* STATE_RINGING      */
@@ -47,27 +37,6 @@ struct state_data_t state_data[] = {
   { "B-Channel open", "Pick up", 0, "Hang up", 1 }  /* STATE_CONVERSATION */
 };
 
-/*
- * Sets new state in session and GUI
- */
-void session_set_state(struct session_t *session, enum state_t state) {
-  session->state = state;
-  gtk_statusbar_pop(GTK_STATUSBAR(session->status_bar),
-		    session->phone_context_id);
-  gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
-		     session->phone_context_id, state_data[state].status_bar);
-
-  gtk_label_set_text(GTK_LABEL(session->pick_up_label),
-		     state_data[state].pick_up_label);
-  gtk_widget_set_sensitive(session->pick_up_button,
-			   state_data[state].pick_up_state);
-
-  gtk_label_set_text(GTK_LABEL(session->hang_up_label),
-		     state_data[state].hang_up_label);
-  gtk_widget_set_sensitive(session->hang_up_button,
-			   state_data[state].hang_up_state);
-}
-
 /*
  * initialize a session (isdn device, audio device)
  *
@@ -184,6 +153,8 @@ int session_init(struct session_t *session,
   /* XXX: read history, later save it */
   session->dial_number_history_maxlen = 10; /* XXX: make configurable */
 
+  session->effect = EFFECT_NONE;
+
   return 0;
 }
 
@@ -247,34 +218,35 @@ int session_deinit(struct session_t *session) {
 }
 
 /*
- * set up handlers for audio input (from ttyI / oss)
+ * set up handlers for audio/ISDN input
  */
 void session_io_handlers_start(struct session_t *session) {
-  session->gdk_isdn_input_tag = gdk_input_add(session->isdn_fd,
-					      GDK_INPUT_READ,
-					      gdk_handle_isdn_input,
-					      (gpointer) session);
   session->gdk_audio_input_tag = gdk_input_add(session->audio_fd_in,
 					       GDK_INPUT_READ,
 					       gdk_handle_audio_input,
 					       (gpointer) session);
+  session->gdk_isdn_input_tag = gdk_input_add(session->isdn_fd,
+					      GDK_INPUT_READ,
+					      gdk_handle_isdn_input,
+					      (gpointer) session);
 }
 
 /*
- * remove handlers for audio input (from ttyI / oss)
+ * remove handlers for audio/ISDN input
  */
 void session_io_handlers_stop(struct session_t *session) {
-  gdk_input_remove(session->gdk_isdn_input_tag);
   gdk_input_remove(session->gdk_audio_input_tag);
+  gdk_input_remove(session->gdk_isdn_input_tag);
 }
 
 /*
  * Resets audio devices by closing and reopening
  *
  * assumes audio in open, initialized (possibly used) state
+ *
+ * WARNING: stop I/O handlers first!
  */
 void session_reset_audio(struct session_t *session) {
-  session_io_handlers_stop(session);
   if (close_audio_devices(session->audio_fd_in, session->audio_fd_out)) {
     fprintf(stderr, "Error closing audio device(s) while resetting.\n");
   }
@@ -292,7 +264,6 @@ void session_reset_audio(struct session_t *session) {
 			 &session->audio_speed_out)) {
     fprintf(stderr, "Error reopening audio device(s) while resetting.\n");
   }
-  session_io_handlers_start(session);
 }
 
 /*
@@ -390,7 +361,9 @@ void session_init_conversation(struct session_t *session) {
  * return: self_hangup: 1 == we hangup, 0 == other side hung up
  */
 void session_deinit_conversation(struct session_t *session, int self_hangup) {
+  session_io_handlers_stop(session);
   session_reset_audio(session);
+  session_io_handlers_start(session);
   
   if (isdn_blockmode(session->isdn_fd, 0))
     fprintf(stderr, "Warning: "
@@ -422,6 +395,7 @@ void session_start_conversation(struct session_t *session) {
     session_set_state(session, STATE_READY);
     cid_call_finished(session, "(HW ERROR)");
   } else { /* full duplex ok: audio mode */
+    session_set_state(session, STATE_CONVERSATION);
     if (isdn_blockmode(session->isdn_fd, 1)) /* blockmode error */
       fprintf(stderr,
 	      "Warning: Switching to ISDN blockmode not successful.\n");
@@ -429,21 +403,7 @@ void session_start_conversation(struct session_t *session) {
     session_init_conversation(session); /* init some conversation variables */
     
     /* start with first block to start recording */
-    process_audio_source(session->audio_fd_in,       /* in-params  */
-			 session->audio_inbuf,
-			 session->fragment_size_in,
-			 
-			 session->isdn_fd,           /* out-params */
-			 session->isdn_outbuf,
-			 session->isdn_outbuf_size,
-			 &session->isdn_outbuf_index,
-			 
-			 session->audio_LUT_out,
-			 session->audio_sample_size_in,
-			 session->ratio_out,
-			 &session->samples_out,
-			 &session->aborted);
-    session_set_state(session, STATE_CONVERSATION);
+    process_audio_source(session);
   }
 }
 
@@ -477,12 +437,18 @@ void session_effect_callback(gpointer data, gint fd,
     seconds = (double)session->effect_pos / session->audio_speed_out;
     switch(session->effect) {
     case EFFECT_RING: /* somebody's calling */
+    case EFFECT_TEST: /* play test sound */
 
-      rest = fmod(seconds, RING_PERIOD);                   /* 4 sec period */
-      rest2 = fmod(seconds, RING_SHORT_PERIOD);            /* short period */
+      if (session->effect == EFFECT_RING) {
+	rest = fmod(seconds, RING_PERIOD);               /* 4 sec period */
+      } else {
+	rest = seconds;                                  /* infinite period */
+      }
+
+      rest2 = fmod(seconds, RING_SHORT_PERIOD);          /* short period */
 
       if (rest < RING_FADE_LENGTH) { /* fade in */
-	factor = -cos(rest * 2 * M_PI / (RING_FADE_LENGTH * 2)) / 2 + 0.5;
+	factor = -cos(rest * M_PI / RING_FADE_LENGTH) / 2 + 0.5;
       } else if (rest > RING_LENGTH - RING_FADE_LENGTH &&
 		 rest < RING_LENGTH) { /* fade out */
 	factor = -cos((RING_LENGTH - rest) * 2 * M_PI / (RING_FADE_LENGTH * 2))
@@ -552,20 +518,78 @@ void session_effect_callback(gpointer data, gint fd,
  * Start an effect (effect_t) playing on the sound device
  */
 void session_effect_start(struct session_t *session, enum effect_t kind) {
-  session->effect = kind;
-  session->effect_pos = 0;
   session->effect_tag = gdk_input_add(session->audio_fd_out,
 				      GDK_INPUT_WRITE,
 				      session_effect_callback,
 				      (gpointer) session);
+
+  session->effect = kind;
+  session->effect_pos = 0;
 }
 
 /*
- * reset sound device and unset callback
+ * Reset sound device and unset callback
  */
 void session_effect_stop(struct session_t *session) {
-  gdk_input_remove(session->effect_tag);
+  if (session->effect != EFFECT_NONE) { /* stop only if already playing */
+    gdk_input_remove(session->effect_tag);
+    session->effect = EFFECT_NONE;
+  }
+  session_io_handlers_stop(session);
   session_reset_audio(session);
+  session_io_handlers_start(session);
+}
+
+/*
+ * Sets new state in session and GUI
+ */
+void session_set_state(struct session_t *session, enum state_t state) {
+  /* some menu items are selected only in STATE_READY */
+  gtk_widget_set_sensitive(session->menuitem_settings, state == STATE_READY);
+  gtk_widget_set_sensitive(session->menuitem_line_check, state == STATE_READY);
+
+  /* start / stop effects when needed */
+  switch (state) {
+  case STATE_DIALING:
+    if (session->effect == EFFECT_NONE)
+      session_effect_start(session, EFFECT_RINGING);
+    break;
+  case STATE_RINGING:
+    if (session->effect == EFFECT_NONE)
+      session_effect_start(session, EFFECT_RING);
+    break;
+  case STATE_READY:
+    gtk_widget_grab_focus(GTK_WIDGET(GTK_COMBO(session->dial_number_box)
+				     ->entry));
+    /* no break */
+  case STATE_CONVERSATION:
+    if (session->effect != EFFECT_NONE)
+      session_effect_stop(session);
+    break;
+  default:
+  }
+
+  /* status line */
+  session->state = state;
+  gtk_statusbar_pop(GTK_STATUSBAR(session->status_bar),
+		    session->phone_context_id);
+  gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
+		     session->phone_context_id, state_data[state].status_bar);
+
+  gtk_label_set_text(GTK_LABEL(session->pick_up_label),
+		     state_data[state].pick_up_label);
+  gtk_widget_set_sensitive(session->pick_up_button,
+			   state_data[state].pick_up_state);
+
+  gtk_label_set_text(GTK_LABEL(session->hang_up_label),
+		     state_data[state].hang_up_label);
+  gtk_widget_set_sensitive(session->hang_up_button,
+			   state_data[state].hang_up_state);
+
+  if (state == STATE_READY) {
+    llcheck_bar_reset(session->llcheck_in);
+    llcheck_bar_reset(session->llcheck_out);
+  }
 }
 
 /*
@@ -583,7 +607,6 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
   case STATE_READY: /* we are in command mode */
     if (isdn_try_read_line(session)){ /* got new line: something happened */
       if (!strncmp(session->isdn_inbuf, "RING/", 5)) { /* -> RINGING state */
-	session_effect_start(session, EFFECT_RING);
 	session_set_state(session, STATE_RINGING);
 	session->isdn_inbuf[session->isdn_inbuf_len - 2] = 0;
 	
@@ -596,12 +619,14 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
   case STATE_DIALING:
     if (isdn_try_read_line(session)){ /* a response to our dial request */
       if (strstr(session->isdn_inbuf, "BUSY\r\n")) { /* get back to READY  */
-	session_effect_stop(session);
 	session_set_state(session, STATE_READY);
 	cid_call_finished(session, "(BUSY)");
       } else if (strstr(session->isdn_inbuf, "VCON\r\n")) { /* let's go! */
-	session_effect_stop(session);
 	session_start_conversation(session); /* including state transition */
+      } else if (strstr(session->isdn_inbuf, "NO CARRIER\r\n")) {
+	/* timeout? */
+	session_set_state(session, STATE_READY);
+	cid_call_finished(session, "(TIMEOUT)");
       } else { /* got some other modem answer string while dialing out */
       }
     }
@@ -609,7 +634,6 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
   case STATE_RINGING:
     if (isdn_try_read_line(session)){ /* got new line: something happened */
       if (strstr(session->isdn_inbuf, "VCON\r\n")) { /* let's go! */
-	/* session_effect_stop(session); - already done before ATA */
 	session_start_conversation(session); /* including state transition */
       } else if (strstr(session->isdn_inbuf, "CALLER NUMBER: ")) {
 	/* got Caller ID */
@@ -618,28 +642,14 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
 	cid_add_data(session, &session->isdn_inbuf[15]);
       } else if (strstr(session->isdn_inbuf, "RUNG\r\n")) {
 	/* caller giving up */
-	session_effect_stop(session);
 	session_set_state(session, STATE_READY);
-	cid_call_finished(session, "--------");
+	cid_call_finished(session, "(RUNG)");
+      } else { /* got some other modem answer string while it rings */
       }
     }
     break;
   case STATE_CONVERSATION:
-    process_isdn_source(session->isdn_fd,             /* in-params  */
-			session->isdn_inbuf,
-			session->isdn_inbuf_size,
-			
-			session->audio_fd_out,      /* out-params */
-			session->audio_outbuf,
-			session->fragment_size_out,
-			&session->audio_outbuf_index,
-			session->audio_LUT_in,
-			session->audio_sample_size_out,
-			&session->escape,
-			session->ratio_in,
-			&session->samples_in,
-			&session->hangup,
-			&session->aborted);
+    process_isdn_source(session);
     if (session->samples_in >= ISDN_SPEED)
       session->samples_in %= ISDN_SPEED;
     session->no_input = 0;
@@ -673,29 +683,28 @@ void gdk_handle_audio_input(gpointer data, gint fd,
 
   switch (session->state) {
   case STATE_READY: /* we are in command mode */
-    fprintf(stderr, "Warning: Got audio input in ready mode.\n");
+    if (debug)
+      fprintf(stderr, "Warning: Got audio input in ready mode.\n");
+    /* flush audio input */
+    read(session->audio_fd_in,
+	 session->audio_inbuf, session->fragment_size_in);
     break;
   case STATE_DIALING:
-    fprintf(stderr, "Warning: Got audio input in dialing mode.\n");
+    if (debug)
+      fprintf(stderr, "Warning: Got audio input in dialing mode.\n");
+    /* flush audio input */
+    read(session->audio_fd_in,
+	 session->audio_inbuf, session->fragment_size_in);
     break;
   case STATE_RINGING:
-    fprintf(stderr, "Warning: Got audio input in ringing mode.\n");
+    if (debug)
+      fprintf(stderr, "Warning: Got audio input in ringing mode.\n");
+    /* flush audio input */
+    read(session->audio_fd_in,
+	 session->audio_inbuf, session->fragment_size_in);
     break;
   case STATE_CONVERSATION:
-    process_audio_source(session->audio_fd_in,       /* in-params  */
-			 session->audio_inbuf,
-			 session->fragment_size_in,
-			 
-			 session->isdn_fd,           /* out-params */
-			 session->isdn_outbuf,
-			 session->isdn_outbuf_size,
-			 &session->isdn_outbuf_index,
-
-			 session->audio_LUT_out,
-			 session->audio_sample_size_in,
-			 session->ratio_out,
-			 &session->samples_out,
-			 &session->aborted);
+    process_audio_source(session);
     if (session->samples_out >= session->audio_speed_in)
       session->samples_out %= session->audio_speed_in;
     session->no_input++;
@@ -751,7 +760,6 @@ void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data) {
 	fprintf(stderr, "Error dialing.\n");
       }
 
-      session_effect_start(session, EFFECT_RINGING);
       session_set_state(session, STATE_DIALING);
 
       /* update dial combo box */
@@ -779,7 +787,6 @@ void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data) {
   case STATE_DIALING: /* already dialing! */
     break;
   case STATE_RINGING: /* we want to pick up the phone while it rings */
-    session_effect_stop(session);
     tty_clear(session->isdn_fd);
     if (tty_write(session->isdn_fd, "ATA\n"))
       fprintf(stderr, "Error answering call.\n");
@@ -807,7 +814,6 @@ void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data) {
     tty_clear(session->isdn_fd);
     if (tty_write(session->isdn_fd, "ATH\n"))
       fprintf(stderr, "Error answering call.\n");
-    session_effect_stop(session);
     session_set_state(session, STATE_READY);
     cid_call_finished(session, "(ABORTED)");
     break;
@@ -815,7 +821,6 @@ void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data) {
     tty_clear(session->isdn_fd);
     if (tty_write(session->isdn_fd, "ATH\n"))
       fprintf(stderr, "Error answering call.\n");
-    session_effect_stop(session);
     session_set_state(session, STATE_READY);
     cid_call_finished(session, "(REJECTED)");
     break;
